@@ -1,7 +1,7 @@
 '''
-    VERIDIC Project - Towards a centralized access control system
+    Authentic 2 - Versatile Identity Server
 
-    Copyright (C) 2011  Mikael Ates
+    Copyright (C) 2011 Entr'ouvert
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Affero General Public License as
@@ -518,6 +518,7 @@ ATTRIBUTE_MAPPING = {
     "oid": "0.9.2342.19200300.100.1.1",
     "display_name": "uid userid",
     "alias": ['userid'],
+    "profile_field_name": 'username',
     "type": "http://www.w3.org/2001/XMLSchema#string",
     "syntax": "1.3.6.1.4.1.1466.115.121.1.15{256}",
 },
@@ -605,6 +606,7 @@ ATTRIBUTE_MAPPING = {
 "displayName": {
     "oid": "2.16.840.1.113730.3.1.241",
     "display_name": "displayName",
+    "profile_field_name": "get_full_name",
     "type": "http://www.w3.org/2001/XMLSchema#string",
     "syntax": "1.3.6.1.4.1.1466.115.121.1.15",
 },
@@ -1327,6 +1329,7 @@ ATTRIBUTE_MAPPING = {
     "oid": "0.9.2342.19200300.100.1.3",
     "display_name": "mail rfc822Mailbox",
     "alias": ['rfc822Mailbox'],
+    "profile_field_name": 'email',
     "type": "http://www.w3.org/2001/XMLSchema#string",
 },
 
@@ -2277,4 +2280,12 @@ ATTRIBUTE_MAPPING = {
     "syntax": "1.3.6.1.4.1.1466.115.121.1.15{128}",
 },
 
+"role": {
+    "oid": "1.3.6.1.4.1.36560.42.1",
+    "display_name": "Role",
+    "profile_field_name": 'roles',
+    "type": "http://www.w3.org/2001/XMLSchema#string",
+    "syntax": "1.3.6.1.4.1.1466.115.121.1.15{128}",
+},
+
 }
