@@ -1,21 +1,6 @@
 from django.contrib import admin
-from django.conf import settings
-
-from models import AttributeItem, AttributeList, AttributePolicy, \
-    UserConsentAttributes
-
-
-class AttributeListAdmin(admin.ModelAdmin):
-    filter_horizontal = ('attributes', )
-    fieldsets = (
-            (None, {
-                'fields' : (
-                    'name',
-                    'attributes',
-                )
-            }),
-    )
 
+from .models import AttributePolicy
 
 class AttributePolicyAdmin(admin.ModelAdmin):
     filter_horizontal = ('source_filter_for_sso_from_push_sources', )
@@ -42,9 +27,4 @@ class AttributePolicyAdmin(admin.ModelAdmin):
 
 
 
-admin.site.register(AttributeItem)
-admin.site.register(AttributeList, AttributeListAdmin)
 admin.site.register(AttributePolicy, AttributePolicyAdmin)
-
-if settings.DEBUG:
-    admin.site.register(UserConsentAttributes)
