@@ -5,6 +5,7 @@ import xml.etree.ElementTree as etree
 import lasso
 from django.core.management.base import BaseCommand, CommandError
 from django.db import transaction
+from django.template.defaultfilters import slugify
 
 from authentic2.saml.models import *
 
@@ -32,12 +33,13 @@ def load_one_entity(tree, options, sp_policy=None, idp_policy=None):
     entity_id = tree.get(ENTITY_ID)
     organization = tree.find(ORGANIZATION)
     name, org = None, None
-    if organization:
+    if organization is not None:
         organization_display_name = organization.find(ORGANIZATION_DISPLAY_NAME)
         organization_name = organization.find(ORGANIZATION_NAME)
-        org = organization_display_name or organization_name
-        if org is not None:
-            name = org.text
+        if organization_display_name is not None:
+            name = organization_display_name.text
+        elif organization_name is not None:
+            name = organization_name.text
     if not name:
         name = entity_id
     idp, sp = False, False
@@ -52,8 +54,9 @@ def load_one_entity(tree, options, sp_policy=None, idp_policy=None):
         print 'Deleted', entity_id
         return
     if idp or sp:
+        slug = slugify(name)
         provider, created = LibertyProvider.objects.get_or_create(entity_id=entity_id,
-                protocol_conformance=3)
+                protocol_conformance=3, defaults={'name': name, 'slug': slug})
         if options['verbosity'] == '2':
             if created:
                 what = 'Creating'
@@ -160,7 +163,7 @@ existing providers with the same tag will be removed if they do not exist\
                     except:
                         print 'SAML2 service provider options policy with name %s not found' % sp_policy_name
                 else:
-                        print 'No SAML2 service provider options policy provided'
+                    print 'No SAML2 service provider options policy provided'
                 idp_policy = None
                 if 'idp_policy' in options and options['idp_policy']:
                     idp_policy_name = options['idp_policy']
@@ -171,9 +174,13 @@ existing providers with the same tag will be removed if they do not exist\
                     except:
                         print 'SAML2 identity provider options policy with name %s not found' % idp_policy_name
                 else:
-                        print 'No SAML2 identity provider options policy provided'
+                    print 'No SAML2 identity provider options policy provided'
                 loaded = []
-                for entity_descriptor in doc.getroot().findall(ENTITY_DESCRIPTOR_TN):
+                if doc.getroot().tag == ENTITY_DESCRIPTOR_TN:
+                    entity_descriptors = [ doc.getroot() ]
+                else:
+                    entity_descriptors = doc.getroot().findall(ENTITY_DESCRIPTOR_TN)
+                for entity_descriptor in entity_descriptors:
                     try:
                         load_one_entity(entity_descriptor, options, sp_policy=sp_policy, idp_policy=idp_policy)
                         loaded.append(entity_descriptor.get(ENTITY_ID))
@@ -181,7 +188,7 @@ existing providers with the same tag will be removed if they do not exist\
                         raise
                         entity_id = entity_descriptor.get(ENTITY_ID)
                         if options['ignore-errors']:
-                            print >>sys.stderr, 'Unable to load EntityDescriptor for %s: %s' % (entity_id, str(e))
+                            print >> sys.stderr, 'Unable to load EntityDescriptor for %s: %s' % (entity_id, str(e))
                         else:
                             raise CommandError('EntityDescriptor loading: %s' % str(e))
                 if options['source']:
