@@ -7,16 +7,16 @@ import datetime
 import lasso
 from django.db import models
 from django.conf import settings
-from django.contrib.auth.models import User
 from django.core.exceptions import ValidationError
-from django.utils.translation import ugettext as _
+from django.utils.translation import ugettext_lazy as _
 from django.core.exceptions import ObjectDoesNotExist
 from django.utils.importlib import import_module
+from django.utils.timezone import now
+from django.dispatch import Signal
 
 from fields import PickledObjectField, MultiSelectField
 
-from authentic2.idp.models import AttributePolicy
-
+from . import app_settings
 
 def metadata_validator(meta):
     provider=lasso.Provider.newFromBuffer(lasso.PROVIDER_ROLE_ANY, meta.encode('utf8'))
@@ -77,24 +77,25 @@ def organization_name(provider):
 ASSERTION_CONSUMER_PROFILES = (
         ('meta', _('Use the default from the metadata file')),
         ('art', _('Artifact binding')),
-        ('post', _('Post binding')))
+        ('post', _('POST binding')))
 
 DEFAULT_NAME_ID_FORMAT = 'none'
 
 # Supported name id formats
 NAME_ID_FORMATS = {
         'none': { 'caption': _('None'),
-            'samlv2': None,
-            'idff12': None },
+            'samlv2': None,},
         'persistent': { 'caption': _('Persistent'),
-            'samlv2': lasso.SAML2_NAME_IDENTIFIER_FORMAT_PERSISTENT,
-            'idff12': lasso.LIB_NAMEID_POLICY_TYPE_FEDERATED },
+            'samlv2': lasso.SAML2_NAME_IDENTIFIER_FORMAT_PERSISTENT,},
         'transient': { 'caption': _("Transient"),
-            'samlv2': lasso.SAML2_NAME_IDENTIFIER_FORMAT_TRANSIENT,
-            'idff12': lasso.LIB_NAMEID_POLICY_TYPE_ONE_TIME },
-        'email': { 'caption': _("Email (only supported by SAMLv2)"),
-            'samlv2': lasso.SAML2_NAME_IDENTIFIER_FORMAT_EMAIL,
-            'idff12': None }}
+            'samlv2': lasso.SAML2_NAME_IDENTIFIER_FORMAT_TRANSIENT,},
+        'email': { 'caption': _("Email"),
+            'samlv2': lasso.SAML2_NAME_IDENTIFIER_FORMAT_EMAIL,},
+        'username': { 'caption': _("Username (use with Google Apps)"),
+            'samlv2': lasso.SAML2_NAME_IDENTIFIER_FORMAT_UNSPECIFIED,},
+        'edupersontargetedid': { 'caption': _("Use eduPersonTargetedID attribute"),
+            'samlv2': lasso.SAML2_NAME_IDENTIFIER_FORMAT_PERSISTENT,}
+}
 
 NAME_ID_FORMATS_CHOICES = \
         tuple([(x, y['caption']) for x, y in NAME_ID_FORMATS.iteritems()])
@@ -112,9 +113,6 @@ def saml2_urn_to_nidformat(urn):
 def nidformat_to_saml2_urn(key):
     return NAME_ID_FORMATS.get(key, {}).get('samlv2')
 
-def nidformat_to_idff12_urn(key):
-    return NAME_ID_FORMATS.get(key, {}).get('idff12')
-
 # According to: saml-profiles-2.0-os
 # The HTTP Redirect binding MUST NOT be used, as the response will typically exceed the URL length permitted by most user agents.
 BINDING_SSO_IDP = (
@@ -134,18 +132,22 @@ SIGNATURE_VERIFY_HINT = {
         lasso.PROFILE_SIGNATURE_VERIFY_HINT_FORCE: _('Always check signatures'),
         lasso.PROFILE_SIGNATURE_VERIFY_HINT_IGNORE: _('Does not check signatures') }
 
-
 class LibertyProviderPolicy(models.Model):
     name = models.CharField(max_length=64, unique=True)
     authn_request_signature_check_hint = models.IntegerField(
             verbose_name=_("How to verify signatures Authentication Request ?"),
             choices=SIGNATURE_VERIFY_HINT.items(),
             default=lasso.PROFILE_SIGNATURE_VERIFY_HINT_MAYBE)
+
     def __unicode__(self):
         options = []
-        options.append('AuthnRequest signature: ' + SIGNATURE_VERIFY_HINT[self.authn_request_signature_check_hint])
+        options.append(u'AuthnRequest signature: %s' % SIGNATURE_VERIFY_HINT[self.authn_request_signature_check_hint])
         return self.name + ' (%s)' % ', '.join(options)
 
+    class Meta:
+        verbose_name = _('liberty service provider policy')
+        verbose_name_plural = _('liberty service provider policies')
+
 
 AUTHSAML2_UNAUTH_PERSISTENT = (
     ('AUTHSAML2_UNAUTH_PERSISTENT_ACCOUNT_LINKING_BY_AUTH',
@@ -178,7 +180,9 @@ class IdPOptionsSPPolicy(models.Model):
             choices = NAME_ID_FORMATS_CHOICES)
     transient_is_persistent = models.BooleanField(
             verbose_name = \
-_("This IdP falsely sends a transient NameID which is in fact persistent"))
+_("This IdP sends a transient NameID but you want a persistent behaviour for your SP"))
+    persistent_identifier_attribute = models.CharField(_('Persistent '
+            'identifier attribute'), max_length=200, null=True, blank=True)
     allow_create = models.BooleanField(
             verbose_name = _("Allow IdP to create an identity"))
     enable_binding_for_sso_response = models.BooleanField(
@@ -200,7 +204,7 @@ _("This IdP falsely sends a transient NameID which is in fact persistent"))
             _('HTTP method for federation termination request \
             (taken from metadata if not enabled)'))
     http_method_for_defederation_request = models.IntegerField(
-            verbose_name = _("HTTP method for the SLO requests"),
+            verbose_name = _("HTTP method for the defederation requests"),
             max_length = 200, choices = HTTP_METHOD,
             default = lasso.HTTP_METHOD_SOAP)
     force_user_consent = models.BooleanField(\
@@ -263,12 +267,12 @@ class SPOptionsIdPPolicy(models.Model):
             verbose_name = _("Allow IdP initiated SSO"))
     # XXX: format in the metadata file, should be suffixed with a star to mark
     # them as special
-    default_name_id_format = models.CharField(max_length = 200,
+    default_name_id_format = models.CharField(max_length = 256,
             default = DEFAULT_NAME_ID_FORMAT,
             choices = NAME_ID_FORMATS_CHOICES)
     accepted_name_id_format = MultiSelectField(
             verbose_name = _("NameID formats accepted"),
-            max_length=ACCEPTED_NAME_ID_FORMAT_LENGTH,
+            max_length=1024,
             blank=True, choices=NAME_ID_FORMATS_CHOICES)
     # TODO: add clean method which checks that the LassoProvider we can create
     # with the metadata file support the SP role
@@ -281,6 +285,9 @@ class SPOptionsIdPPolicy(models.Model):
     forward_slo = models.BooleanField(\
             verbose_name = _("Forward Single Logout requests"),
             default=True)
+    federation_mode = models.PositiveIntegerField(_('federation mode'),
+            choices=app_settings.FEDERATION_MODE.get_choices(app_settings),
+            default=app_settings.FEDERATION_MODE.get_default(app_settings))
 
     class Meta:
         verbose_name = _('service provider options policy')
@@ -295,6 +302,10 @@ class AuthorizationAttributeMap(models.Model):
     def __unicode__(self):
         return self.name
 
+    class Meta:
+        verbose_name = _('authorization attribute map')
+        verbose_name_plural = _('authorization attribute maps')
+
 class AuthorizationAttributeMapping(models.Model):
     source_attribute_name = models.CharField(max_length = 40,
             blank=True)
@@ -304,6 +315,10 @@ class AuthorizationAttributeMapping(models.Model):
     attribute_value = models.CharField(max_length = 40)
     map = models.ForeignKey(AuthorizationAttributeMap)
 
+    class Meta:
+        verbose_name = _('authorization attribute mapping')
+        verbose_name_plural = _('authorization attribute mappings')
+
 class AuthorizationSPPolicy(models.Model):
     name = models.CharField(_('name'), max_length=80, unique=True)
     enabled = models.BooleanField(verbose_name = _('Enabled'))
@@ -317,23 +332,27 @@ class AuthorizationSPPolicy(models.Model):
             default=_('You are not authorized to access the service.'))
 
     class Meta:
-        verbose_name = _('authorization policy')
-        verbose_name_plural = _('authorization policies')
+        verbose_name = _('authorization identity providers policy')
+        verbose_name_plural = _('authorization identity providers policies')
 
     def __unicode__(self):
         return self.name
 
+
+class GetBySlugManager(models.Manager):
+    def get_by_natural_key(self, slug):
+        return self.get(slug=slug)
+
+
 class LibertyProvider(models.Model):
-    entity_id = models.URLField(unique = True)
-    entity_id_sha1 = models.CharField(max_length = 40, blank=True)
     name = models.CharField(max_length = 140,
             help_text = _("Internal nickname for the service provider"),
             blank = True)
+    slug = models.SlugField(max_length=140, unique=True)
+    entity_id = models.URLField(unique = True)
+    entity_id_sha1 = models.CharField(max_length = 40, blank=True)
     protocol_conformance = models.IntegerField(max_length = 10,
-            choices = ((0, 'SAML 1.0'),
-                       (1, 'SAML 1.1'),
-                       (2, 'SAML 1.2'),
-                       (3, 'SAML 2.0')))
+            choices = ((lasso.PROTOCOL_SAML_2_0, 'SAML 2.0'),))
     metadata = models.TextField(validators = [ metadata_validator ])
     # All following field must be PEM formatted textual data
     public_key = models.TextField(blank=True)
@@ -341,6 +360,8 @@ class LibertyProvider(models.Model):
     ca_cert_chain = models.TextField(blank=True)
     federation_source = models.CharField(max_length=64, blank=True, null=True)
 
+    objects = GetBySlugManager()
+
     def __unicode__(self):
         return self.name
 
@@ -368,11 +389,36 @@ class LibertyProvider(models.Model):
     def clean(self):
         super(LibertyProvider, self).clean()
         p = lasso.Provider.newFromBuffer(lasso.PROVIDER_ROLE_ANY, self.metadata.encode('utf8'))
+        if p is None:
+            raise ValidationError(_('Invalid metadata file'))
         self.entity_id = p.providerId
         if not self.name:
             self.name = organization_name(p)
-        if not self.protocol_conformance:
-            self.protocol_conformance = p.protocolConformance
+        self.protocol_conformance = p.protocolConformance
+        if self.protocol_conformance != lasso.PROTOCOL_SAML_2_0:
+            raise ValidationError(_('Protocol other than SAML 2.0 are unsupported'))
+
+    def natural_key(self):
+        return (self.slug,)
+
+    class Meta:
+        ordering = ('name',)
+        verbose_name = _('liberty provider')
+        verbose_name_plural = _('liberty providers')
+
+def get_all_custom_or_default(instance, name):
+    model = instance._meta.get_field_by_name(name)[0].rel.to
+    try:
+        return model.objects.get(name='All')
+    except ObjectDoesNotExist:
+        pass
+    custom = getattr(instance, name, None)
+    if custom is not None:
+        return custom
+    try:
+        return models.objects.get(name='Default')
+    except ObjectDoesNotExist:
+        raise RuntimeError('Default %s is missing' % model)
 
 # TODO: The IdP must look to the preferred binding order for sso in the SP metadata (AssertionConsumerService)
 # expect if the protocol for response is defined in the request (ProtocolBinding attribute)
@@ -382,14 +428,24 @@ class LibertyServiceProvider(models.Model):
     enabled = models.BooleanField(verbose_name = _('Enabled'))
     enable_following_sp_options_policy = models.BooleanField(verbose_name = \
         _('The following options policy will apply except if a policy for all service provider is defined.'))
-    sp_options_policy = models.ForeignKey(SPOptionsIdPPolicy, related_name = "sp_options_policy", verbose_name = _('SP Options Policy'), blank=True, null=True)
+    sp_options_policy = models.ForeignKey(SPOptionsIdPPolicy, related_name = "sp_options_policy", verbose_name = _('service provider options policy'), blank=True, null=True)
     policy = models.ForeignKey(LibertyProviderPolicy,
             verbose_name=_("Protocol policy"), null=True, default=1)
     enable_following_attribute_policy = models.BooleanField(verbose_name = \
         _('The following attribute policy will apply except if a policy for all service provider is defined.'))
-    attribute_policy = models.ForeignKey(AttributePolicy,
+    attribute_policy = models.ForeignKey('idp.AttributePolicy',
              related_name = "attribute_policy",
-            verbose_name=_("Attribute policy"), null=True, blank=True)
+            verbose_name=_("attribute policy"), null=True, blank=True)
+
+    def get_policy(self):
+        return get_all_custom_or_default(self, 'policy')
+
+    def __unicode__(self):
+        return unicode(self.liberty_provider)
+
+    class Meta:
+        verbose_name = _('liberty service provider')
+        verbose_name_plural = _('liberty service providers')
 
 
 # TODO: The choice for requests must be restricted by the IdP metadata
@@ -401,25 +457,29 @@ class LibertyIdentityProvider(models.Model):
             primary_key = True, related_name = 'identity_provider')
     enabled = models.BooleanField(verbose_name = _('Enabled'))
     enable_following_idp_options_policy = models.BooleanField(verbose_name = \
-        _('The following options policy will apply except if a policy for all identity provider is defined.'))
-    idp_options_policy = models.ForeignKey(IdPOptionsSPPolicy, related_name = "idp_options_policy", verbose_name = _('IdP Options Policy'), blank=True, null=True)
+        _('The following options policy will apply except if a policy for all '
+          'identity provider is defined.'))
+    idp_options_policy = models.ForeignKey(IdPOptionsSPPolicy,
+            related_name="idp_options_policy",
+            verbose_name=_('identity provider options policy'), blank=True,
+            null=True)
     enable_following_authorization_policy = models.BooleanField(verbose_name = \
-        _('The following authorization policy will apply except if a policy for all identity provider is defined.'))
-    authorization_policy = models.ForeignKey(AuthorizationSPPolicy, related_name = "authorization_policy", verbose_name = _('Authorization Policy'), blank=True, null=True)
+        _('The following authorization policy will apply except if a policy for'
+          ' all identity provider is defined.'))
+    authorization_policy = models.ForeignKey(AuthorizationSPPolicy,
+            related_name="authorization_policy",
+            verbose_name=_('authorization identity providers policy'), blank=True, null=True)
 
     # TODO: add clean method which checks that the LassoProvider we can create
     # with the metadata file support the IDP role
     # i.e. provider.roles & lasso.PROVIDER_ROLE_IDP != 0
 
+    def __unicode__(self):
+        return unicode(self.liberty_provider)
 
-# Transactional models
-class LibertyIdentityDump(models.Model):
-    '''Store lasso identity dump
-
-       Should be replaced in the future by direct reference to LassoFederation
-       objects'''
-    user = models.ForeignKey(User, unique = True)
-    identity_dump = models.TextField(blank = True)
+    class Meta:
+        verbose_name = _('liberty identity provider')
+        verbose_name_plural = _('liberty identity providers')
 
 class SessionLinkedManager(models.Manager):
     def cleanup(self):
@@ -429,11 +489,6 @@ class SessionLinkedManager(models.Manager):
             key = o.django_session_key
             if not store.exists(key):
                 o.delete()
-            else:
-                session = engine.SessionStore(session_key=key)
-                if session.get_expiry_date() >= datetime.datetime.now():
-                    store.delete(key)
-                    o.delete()
 
 LIBERTY_SESSION_DUMP_KIND_SP = 0
 LIBERTY_SESSION_DUMP_KIND_IDP = 1
@@ -445,41 +500,52 @@ class LibertySessionDump(models.Model):
 
        Should be replaced in the future by direct references to known
        assertions through the LibertySession object'''
-    django_session_key = models.CharField(max_length = 40)
+    django_session_key = models.CharField(max_length = 128)
     session_dump = models.TextField(blank = True)
     kind = models.IntegerField(choices = LIBERTY_SESSION_DUMP_KIND.items())
 
     objects = SessionLinkedManager()
 
+    class Meta:
+        verbose_name = _('liberty session dump')
+        verbose_name_plural = _('liberty session dumps')
+
 class LibertyManageDump(models.Model):
     '''Store lasso manage dump
 
        Should be replaced in the future by direct reference to ?
        objects'''
-    django_session_key = models.CharField(max_length = 40)
+    django_session_key = models.CharField(max_length = 128)
     manage_dump = models.TextField(blank = True)
 
     objects = SessionLinkedManager()
 
+    class Meta:
+        verbose_name = _('liberty manage dump')
+        verbose_name_plural = _('liberty manage dumps')
+
 class LibertyArtifactManager(models.Manager):
     def cleanup(self):
-        # keep artifacts 10 minutes
         expire = getattr(settings, 'SAML2_ARTIFACT_EXPIRATION', 600)
-        before = datetime.datetime.now()-datetime.timedelta(seconds=expire)
+        before = now()-datetime.timedelta(seconds=expire)
         self.filter(creation__lt=before).delete()
 
 class LibertyArtifact(models.Model):
     """Store an artifact and the associated XML content"""
     creation = models.DateTimeField(auto_now_add=True)
-    artifact = models.CharField(max_length = 40, primary_key = True)
+    artifact = models.CharField(max_length = 128, primary_key = True)
     content = models.TextField()
-    django_session_key = models.CharField(max_length = 40)
-    provider_id = models.CharField(max_length = 80)
+    provider_id = models.CharField(max_length = 256)
 
     objects = LibertyArtifactManager()
 
+    class Meta:
+        verbose_name = _('liberty artifact')
+        verbose_name_plural = _('liberty artifacts')
+
 def nameid2kwargs(name_id):
-    return { 'name_id_qualifier': name_id.nameQualifier,
+    return {
+        'name_id_qualifier': name_id.nameQualifier,
         'name_id_sp_name_qualifier': name_id.spNameQualifier,
         'name_id_content': name_id.content,
         'name_id_format': name_id.format }
@@ -488,13 +554,13 @@ class LibertyAssertionManager(models.Manager):
     def cleanup(self):
         # keep assertions 1 week
         expire = getattr(settings, 'SAML2_ASSERTION_EXPIRATION', 3600*24*7)
-        before = datetime.datetime.now()-datetime.timedelta(seconds=expire)
+        before = now()-datetime.timedelta(seconds=expire)
         self.filter(creation__lt=before).delete()
 
 class LibertyAssertion(models.Model):
-    assertion_id = models.CharField(max_length = 50)
-    provider_id = models.CharField(max_length = 80)
-    session_index = models.CharField(max_length = 80, )
+    assertion_id = models.CharField(max_length = 128)
+    provider_id = models.CharField(max_length = 256)
+    session_index = models.CharField(max_length = 128, )
     assertion = models.TextField()
     creation = models.DateTimeField(auto_now_add=True)
 
@@ -507,22 +573,43 @@ class LibertyAssertion(models.Model):
             kwargs['assertion'] = saml2_assertion.exportToXml()
         models.Model.__init__(self, *args, **kwargs)
 
+    class Meta:
+        verbose_name = _('liberty assertion')
+        verbose_name_plural = _('liberty assertions')
+
+
+federation_delete = Signal()
+
+class LibertyFederationManager(models.Manager):
+    def cleanup(self):
+        for federation in self.filter(user__isnull=True):
+            results = federation_delete.send_robust(sender=federation)
+            for callback, result in results:
+                if not result:
+                    return
+            federation.delete()
+
+
 class LibertyFederation(models.Model):
     """Store a federation, i.e. an identifier shared with another provider, be
        it IdP or SP"""
-    user = models.ForeignKey(User)
-    idp_id = models.CharField(max_length=80)
-    sp_id = models.CharField(max_length=80)
-    name_id_qualifier = models.CharField(max_length = 150,
-            verbose_name = "NameQualifier", blank=True, null=True)
+    user = models.ForeignKey(settings.AUTH_USER_MODEL, null=True, blank=True,
+            on_delete=models.SET_NULL)
+    idp = models.ForeignKey('LibertyIdentityProvider', null=True, blank=True)
+    sp = models.ForeignKey('LibertyServiceProvider', null=True, blank=True)
     name_id_format = models.CharField(max_length = 100,
             verbose_name = "NameIDFormat", blank=True, null=True)
     name_id_content = models.CharField(max_length = 100,
             verbose_name = "NameID")
-    name_id_sp_name_qualifier = models.CharField(max_length = 100,
+    name_id_qualifier = models.CharField(max_length = 256,
+            verbose_name = "NameQualifier", blank=True, null=True)
+    name_id_sp_name_qualifier = models.CharField(max_length = 256,
             verbose_name = "SPNameQualifier", blank=True, null=True)
-    name_id_sp_provided_id = models.CharField(max_length=100,
-            verbose_name="SPProvidedID", blank=True, null=True)
+    termination_notified = models.BooleanField(blank=True, default=False)
+    creation = models.DateTimeField(auto_now_add=True)
+    last_modification = models.DateTimeField(auto_now=True)
+
+    objects = LibertyFederationManager()
 
     def __init__(self, *args, **kwargs):
         saml2_assertion = kwargs.pop('saml2_assertion', None)
@@ -531,32 +618,40 @@ class LibertyFederation(models.Model):
             kwargs.update(nameid2kwargs(name_id))
         models.Model.__init__(self, *args, **kwargs)
 
+    def is_unique(self, for_format=True):
+        '''Return whether a federation already exist for this user and this provider.
+
+           By default the check is made by name_id_format, if you want to check
+           whatever the format, set for_format to False.
+        '''
+        qs = LibertyFederation.objects.exclude(id=self.id) \
+                .filter(user=self.user, idp=self.idp, sp=self.sp)
+        if for_format:
+            qs = qs.filter(name_id_format=self.name_id_format)
+        return not qs.exists()
+
     class Meta:
-        verbose_name = _("Liberty federation")
-        verbose_name_plural = _("Liberty federations")
-        # XXX: To allow shared-federation (multiple-user with the same
-        # federation), add user to this list
-        unique_together = (("name_id_qualifier", "name_id_format",
-            "name_id_content", "name_id_sp_name_qualifier"))
+        verbose_name = _("liberty federation")
+        verbose_name_plural = _("liberty federations")
 
     def __unicode__(self):
-        return '<LibertyFederation %s>' % self.__dict__
+        return self.name_id_content
 
 
 class LibertySession(models.Model):
     """Store the link between a Django session and a Liberty session"""
-    django_session_key = models.CharField(max_length = 40)
+    django_session_key = models.CharField(max_length = 128)
     session_index = models.CharField(max_length = 80)
-    provider_id = models.CharField(max_length = 80)
+    provider_id = models.CharField(max_length = 256)
     federation = models.ForeignKey(LibertyFederation, null = True)
     assertion = models.ForeignKey(LibertyAssertion, null = True)
-    name_id_qualifier = models.CharField(max_length = 150,
+    name_id_qualifier = models.CharField(max_length = 256,
             verbose_name = _("Qualifier"), null = True)
     name_id_format = models.CharField(max_length = 100,
             verbose_name = _("NameIDFormat"), null = True)
     name_id_content = models.CharField(max_length = 100,
             verbose_name = _("NameID"))
-    name_id_sp_name_qualifier = models.CharField(max_length = 100,
+    name_id_sp_name_qualifier = models.CharField(max_length = 256,
             verbose_name = _("SPNameQualifier"), null = True)
     creation = models.DateTimeField(auto_now_add=True)
 
@@ -577,24 +672,37 @@ class LibertySession(models.Model):
     @classmethod
     def get_for_nameid_and_session_indexes(cls, name_id, session_indexes):
         kwargs = nameid2kwargs(name_id)
-        return LibertySession.objects.filter(session_index__in = session_indexes, **kwargs)
+        return LibertySession.objects.filter(session_index__in=session_indexes,
+                **kwargs)
 
     def __unicode__(self):
         return '<LibertySession %s>' % self.__dict__
 
+    class Meta:
+        verbose_name = _("liberty session")
+        verbose_name_plural = _("liberty sessions")
+
 class LibertySessionSP(models.Model):
     """Store the link between a Django session and a Liberty session on the SP"""
-    django_session_key = models.CharField(max_length = 40)
+    django_session_key = models.CharField(max_length = 128)
     session_index =  models.CharField(max_length = 80, )
     federation = models.ForeignKey(LibertyFederation)
 
+    class Meta:
+        verbose_name = _("liberty service provider session")
+        verbose_name_plural = _("liberty service provider sessions")
+
 class KeyValue(models.Model):
-    key = models.CharField(max_length=40, primary_key=True)
+    key = models.CharField(max_length=128, primary_key=True)
     value = PickledObjectField()
 
     def __unicode__(self):
         return self.key
 
+    class Meta:
+        verbose_name = _("key value association")
+        verbose_name_plural = _("key value associations")
+
 def save_key_values(key, *values):
     KeyValue(key = key, value = values).save()
 
